import arrow
import json
import logging
import os
import sxtwl
from jinja2 import Template

from .const import GANS
from .const import GUA5
from .const import GUA64
from .const import GUAS
from .const import XING5
from .const import YAOS
from .const import ZHI5
from .const import ZHIS
from .utils import GZ5X
from .utils import God6
from .utils import Qin6
from .utils import getNajia
from .utils import get_type
from .utils import palace
from .utils import setShiYao
from .utils import xkong

logging.basicConfig(level='INFO')
logger = logging.getLogger(__name__)


class Najia(object):
    bian = None  # 变卦
    hide = None  # 伏神
    data = None

    @staticmethod
    def _gz(cal):
        """
        获取干支
        :param cal:
        :return:
        """
        return GANS[cal.tg] + ZHIS[cal.dz]

    @staticmethod
    def _cn(cal):
        """
        转换中文干支
        :param cal:
        :return:
        """
        return GANS[cal.tg] + ZHIS[cal.dz]

    def _daily(self, date=None):
        """
        计算日期
        :param date:
        :return:
        """
        lunar = sxtwl.Lunar()
        daily = lunar.getDayBySolar(date.year, date.month, date.day)
        hour = lunar.getShiGz(daily.Lday2.tg, date.hour)

        return {
            'xkong': xkong(''.join([GANS[daily.Lday2.tg], ZHIS[daily.Lday2.dz]])),
            'month': daily.Lmonth2,
            'year' : daily.Lyear2,
            'day'  : daily.Lday2,
            'hour' : hour,
            'gz'   : {
                'month': self._gz(daily.Lmonth2),
                'year' : self._gz(daily.Lyear2),
                'day'  : self._gz(daily.Lday2),
                'hour' : self._gz(hour),
            },
            'cn'   : {
                'month': self._gz(daily.Lmonth2),
                'year' : self._gz(daily.Lyear2),
                'day'  : self._gz(daily.Lday2),
                'hour' : self._gz(hour),
            }
        }

    @staticmethod
    def _hidden(gong=None, qins=None):
        """
        计算伏神卦

        :param gong:
        :param qins:
        :return:
        """
        if gong is None:
            raise Exception('')

        if qins is None:
            raise Exception('')

        if len(set(qins)) < 5:
            mark = YAOS[gong] * 2

            logger.debug(mark)

            # 六亲
            qin6 = [(Qin6(XING5[int(GUA5[gong])], ZHI5[ZHIS.index(x[1])])) for x in getNajia(mark)]

            # 干支五行
            qinx = [GZ5X(x) for x in getNajia(mark)]
            seat = [qin6.index(x) for x in list(set(qin6).difference(set(qins)))]

            return {
                'name': GUA64.get(mark),
                'mark': mark,
                'qin6': qin6,
                'qinx': qinx,
                'seat': seat,
            }

        return None

    @staticmethod
    def _transform(params=None, gong=None):
        """
        计算变卦

        :param params:
        :return:
        """

        if params is None:
            raise Exception('')

        if type(params) == str:
            params = [x for x in params]

        if len(params) < 6:
            raise Exception('')

        if 3 in params or 4 in params:
            mark = ''.join(['1' if v in [1, 4] else '0' for v in params])
            qin6 = [(Qin6(XING5[int(GUA5[gong])], ZHI5[ZHIS.index(x[1])])) for x in getNajia(mark)]
            qinx = [GZ5X(x) for x in getNajia(mark)]

            return {
                'name': GUA64.get(mark),
                'mark': mark,
                'qin6': qin6,
                'qinx': qinx,
                'gong': GUAS[palace(mark, setShiYao(mark)[0])],
            }

        return None

    def compile(self, params=None, gender=1, date=None, title=None, guaci=False):
        """
        根据参数编译卦

        :param guaci:
        :param title:
        :param gender:
        :param params:
        :param date:
        :return:
        """
        solar = arrow.now() if date is None else arrow.get(date)
        lunar = self._daily(solar)

        gender = '男' if gender == 1 else '女'

        # 卦码
        mark = ''.join([str(int(l) % 2) for l in params])

        # logger.debug(mark)

        shiy = setShiYao(mark)  # 世应爻

        # 卦宫
        gong = palace(mark, shiy[0])  # 卦宫

        # 卦名
        name = GUA64[mark]

        # 六亲
        qin6 = [(Qin6(XING5[int(GUA5[gong])], ZHI5[ZHIS.index(x[1])])) for x in getNajia(mark)]
        qinx = [GZ5X(x) for x in getNajia(mark)]

        # logger.debug(qin6)

        # 六神
        god6 = God6(''.join([GANS[lunar['day'].tg], ZHIS[lunar['day'].dz]]))

        # 动爻位置
        dong = [i for i, x in enumerate(params) if x > 2]
        # logger.debug(dong)

        # 伏神
        hide = self._hidden(gong, qin6)

        # 变卦
        bian = self._transform(params=params, gong=gong)

        self.data = {
            'params': params,
            'gender': gender,
            'title' : title,
            'guaci' : guaci,
            'solar' : solar,
            'lunar' : lunar,
            'god6'  : god6,
            'dong'  : dong,
            'name'  : name,
            'mark'  : mark,
            'gong'  : GUAS[gong],
            'shiy'  : shiy,
            'qin6'  : qin6,
            'qinx'  : qinx,
            'bian'  : bian,
            'hide'  : hide,
        }

        # logger.debug(self.data)

        return self

    def gua_type(self, i):
        return

    def render(self):
        """

        :return:
        """
        tpl = '''
{{gender}}测：{{title}}

公历：{{solar.year}}年 {{solar.month}}月 {{solar.day}}日 {{solar.hour}}时 {{solar.minute}}分
干支：{{lunar.gz.year}}年 {{lunar.gz.month}}月 {{lunar.gz.day}}日 {{lunar.gz.hour}}时 （旬空：{{lunar.xkong}})

得「{{name}}」{% if bian.name %}之「{{bian.name}}」{% endif %}卦

{{main.indent}}{{main.gong}}宫: {{main.name}}{% if main.type %} ({{main.type}}){% endif %}{{bian.indent}}{% if bian.name %}{{bian.gong}}宫: {{bian.name}}{% if bian.type %} ({{bian.type}}){% endif %}{% endif %}
{{god6.5}}{{hide.qin6.5}}{{qin6.5}}{{qinx.5}} {{main.mark.5}} {{shiy.5}} {{dyao.5}} {{bian.qin6.5}} {{bian.mark.5}}
{{god6.4}}{{hide.qin6.4}}{{qin6.4}}{{qinx.4}} {{main.mark.4}} {{shiy.4}} {{dyao.4}} {{bian.qin6.4}} {{bian.mark.4}}
{{god6.3}}{{hide.qin6.3}}{{qin6.3}}{{qinx.3}} {{main.mark.3}} {{shiy.3}} {{dyao.3}} {{bian.qin6.3}} {{bian.mark.3}}
{{god6.2}}{{hide.qin6.2}}{{qin6.2}}{{qinx.2}} {{main.mark.2}} {{shiy.2}} {{dyao.2}} {{bian.qin6.2}} {{bian.mark.2}}
{{god6.1}}{{hide.qin6.1}}{{qin6.1}}{{qinx.1}} {{main.mark.1}} {{shiy.1}} {{dyao.1}} {{bian.qin6.1}} {{bian.mark.1}}
{{god6.0}}{{hide.qin6.0}}{{qin6.0}}{{qinx.0}} {{main.mark.0}} {{shiy.0}} {{dyao.0}} {{bian.qin6.0}} {{bian.mark.0}}

{% if guaci %}{{ guaci }}{% endif %}'''

        empty = '\u3000' * 6
        rows = self.data

        yaos = ['━　━', '━━━━', '━　━', '○→', '×→']
        # yaos = ['▅▅  ▅▅', '▅▅▅▅▅▅', '▅▅  ▅▅', '○→', '×→']
        rows['dyao'] = [yaos[x] if x in (3, 4) else '' for x in self.data['params']]

        rows['main'] = {}
        rows['main']['mark'] = [yaos[int(x)] for x in self.data['mark']]
        rows['main']['type'] = get_type(self.data['mark'])

        rows['main']['gong'] = rows['gong']
        rows['main']['name'] = rows['name']
        rows['main']['indent'] = '\u3000' * 2

        if rows.get('hide'):
            rows['hide']['qin6'] = [' %s%s ' % (rows['hide']['qin6'][x], rows['hide']['qinx'][x]) if x in rows['hide']['seat'] else empty for x in range(0, 6)]
            rows['main']['indent'] += empty
        else:
            rows['main']['indent'] += '\u3000' * 1
            rows['hide'] = {'qin6': ['  ' for _ in range(0, 6)]}

        rows['main']['display'] = "{indent}{name} ({gong}-{type})".format(**rows['main'])

        if rows.get('bian'):
            if rows.get('hide'):
                s = 19
            else:
                s = 8

            rows['bian']['type'] = get_type(rows['bian']['mark'])
            rows['bian']['indent'] = (s - len(rows['main']['display'])) * '\u3000'

            if rows['bian']['qin6']:
                # 变卦六亲问题
                rows['bian']['qin6'] = [f'{rows["bian"]["qin6"][x]}{rows["bian"]["qinx"][x]}' if x in self.data['dong'] else f'  {rows["bian"]["qin6"][x]}{rows["bian"]["qinx"][x]}'
                                        for x in range(0, 6)]

            if rows['bian']['mark']:
                rows['bian']['mark'] = [x for x in rows['bian']['mark']]
                rows['bian']['mark'] = [yaos[int(rows['bian']['mark'][x])] for x in range(0, 6)]
        else:
            rows['bian'] = {'qin6': [' ' for _ in range(0, 6)], 'mark': [' ' for _ in range(0, 6)]}

        shiy = []

        # 显示世应字
        for x in range(0, 6):
            if x == self.data['shiy'][0] - 1:
                shiy.append('世')
            elif x == self.data['shiy'][1] - 1:
                shiy.append('应')
            else:
                shiy.append('  ')

        rows['shiy'] = shiy

        if self.data['guaci']:
            rows['guaci'] = json.load(open(os.path.join(os.path.dirname(__file__), 'data/dd.json'))).get(rows['name'])
            rows['guaci'] = rows.get('guaci', '').replace('********************', '').replace('　象曰：', '象曰：')

        template = Template(tpl)
        return template.render(**rows)

    def export(self):
        solar, params = self.data
        return solar, params

    def predict(self):
        return



import random
import sys

import click

from . import __version__
from .najia import Najia


@click.command()
@click.help_option('-h', '--help')
@click.version_option(__version__, "-V", "--version", prog_name="Najia", message="%(prog)s: version %(version)s", )
@click.option('-p', '--params', default=None, help='摇卦参数')
@click.option('-g', '--gender', default=1, help='摇卦人性别.')
@click.option('-l', '--lunar', default=False, help='是否阴历.')
@click.option('-t', '--title', default='', help='求卦问卜事情.')
@click.option('-c', '--guaci', is_flag=True, help='是否显示卦辞.')
@click.option('-d', '--date', default=None, help='日期 YYYY-MM-DD hh:mm.')
def main(params, gender, lunar, date, title, guaci):
    params = [random.randint(1, 4) for _ in range(0, 6)] if params is None else params
    params = [int(x) for x in params.replace(',', '')] if type(params) == str else params

    gua = Najia().compile(params=params, gender=gender, date=date, title=title, guaci=guaci)
    res = gua.render()

    print(res)

    return 0


if __name__ == '__main__':
    sys.exit(main())  # pragma: no cover



import logging
import math

from . import const

logging.basicConfig(level='INFO')
logger = logging.getLogger(__name__)


def GZ5X(gz=''):
    """
    干支五行
    :param gz:
    :return:
    """
    _, z = [i for i in gz]
    zm = const.ZHIS.index(z)

    return gz + const.XING5[const.ZHI5[zm]]


def mark(symbol=None):
    """
    单拆重交 转 二进制卦码
    :param symbol:
    :return:
    """

    res = [str(int(x) % 2) for x in symbol]
    logger.debug(res)

    return res


def xkong(gz='甲子'):
    """
    计算旬空

    :param gz: 甲子 or 3,11
    :return:
    """

    gm, zm = [i for i in gz]

    if type(gz) == str:
        gm = const.GANS.index(gm)
        zm = const.ZHIS.index(zm)

    if gm == zm or zm < gm:
        zm += 12

    xk = int((zm - gm) / 2) - 1

    return const.KONG[xk]


def God6(gz=None):
    """
    # 六神, 根据日干五行配对六神五行

    :param gz: 日干支
    :return:
    """

    gm, _ = [i for i in gz]

    if type(gm) is str:
        gm = const.GANS.index(gm)

    num = math.ceil((gm + 1) / 2) - 7

    if gm == 4:
        num = -4

    if gm == 5:
        num = -3

    if gm > 5:
        num += 1

    return const.SHEN6[num:] + const.SHEN6[:num]


'''
寻世诀：
天同二世天变五，地同四世地变初。
本宫六世三世异，人同游魂人变归。

1. 天同人地不同世在二，天不同人地同在五
2. 三才不同世在三
3. 人同其他不同世在四，人不同其他同在三'''


# 世爻初爻是1，二爻是2
# 寻世诀： 天同二世天变五  地同四世地变初  本宫六世三世异  人同游魂人变归
# int('111', 2) => 7
# 世爻 >= 3, 应爻 = 世爻 - 3， index = 5 - 世爻 + 1
# 世爻 <= 3, 应爻 = 世爻 + 3，
# life oneself
def setShiYao(symbol=None):
    """
    获取世爻

    :param symbol: 卦的二进制码
    :return: 世爻，应爻，所在卦宫位置
    """
    wai = symbol[3:]  # 外卦
    nei = symbol[:3]  # 内卦

    def shiy(shi, index=None):
        ying = shi - 3 if shi > 3 else shi + 3
        index = shi if index is None else index
        return shi, ying, index

    # 天同二世天变五
    if wai[2] == nei[2]:
        if wai[1] != nei[1] and wai[0] != nei[0]:
            return shiy(2)
    else:
        if wai[1] == nei[1] and wai[0] == nei[0]:
            return shiy(5)

    # 人同游魂人变归
    if wai[1] == nei[1]:
        if wai[0] != nei[0] and wai[2] != nei[2]:
            return shiy(4, 6)  # , Hun
    else:
        # fix 归魂问题
        if wai[0] == nei[0] and wai[2] == nei[2]:
            return shiy(3, 6)  # , Hun

    # 地同四世地变初
    if wai[0] == nei[0]:
        if wai[1] != nei[1] and wai[2] != nei[2]:
            return shiy(4)
    else:
        if wai[1] == nei[1] and wai[2] == nei[2]:
            return shiy(1)

    # 本宫六世
    if wai == nei:
        return shiy(6)

    # 三世异
    return shiy(3)


def get_type(symbol=None):
    if res := soul(symbol):
        return res

    if attack(symbol):
        return '六冲'

    if res := unite(symbol):
        return res

    return ''


def unite(symbol=None):
    name = const.GUA64[symbol]

    for x in const.LIUHE:
        if x in name:
            return '六合'

    return None


def soul(symbol=None):
    wai = symbol[3:]  # 外卦
    nei = symbol[:3]  # 内卦
    hun = ''

    if wai[1] == nei[1]:
        if wai[0] != nei[0] and wai[2] != nei[2]:
            hun = '游魂'
    else:
        if wai[0] == nei[0] and wai[2] == nei[2]:
            hun = '归魂'

    return hun


def palace(symbol=None, index=None):  # inStr -> '111000'  # intNum -> 世爻
    """
    六爻卦的卦宫名

    认宫诀：
    一二三六外卦宫，四五游魂内变更。
    若问归魂何所取，归魂内卦是本宫。

    :param symbol: 卦的二进制码
    :param index: 世爻
    :return:
    """

    wai = symbol[3:]  # 外卦
    nei = symbol[:3]  # 内卦
    hun = ''

    if wai[1] == nei[1]:
        if wai[0] != nei[0] and wai[2] != nei[2]:
            hun = '游魂'
    else:
        if wai[0] == nei[0] and wai[2] == nei[2]:
            hun = '归魂'

    # 归魂内卦是本宫
    if hun == '归魂':
        return const.YAOS.index(nei)

    # 一二三六外卦宫
    if index in (1, 2, 3, 6):
        return const.YAOS.index(wai)

    # 四五游魂内变更
    if index in (4, 5) or hun == '游魂':
        symbol = ''.join([str(int(c) ^ 1) for c in nei])
        return const.YAOS.index(symbol)


# 判断是否六冲卦
# verb
def attack(symbol):
    wai = symbol[3:]  # 外卦
    nei = symbol[:3]  # 内卦

    # 内外卦相同
    if wai == nei:
        return True

    # 天雷无妄 和 雷天大壮
    gua = [nei, wai]

    try:
        if len(set(gua).difference(('100', '111'))) == 0:
            return True
    except TypeError:
        pass

    return False


# 纳甲配干支
def getNajia(symbol=None):
    """
    纳甲配干支

    :param symbol:
    :return:
    """

    wai = symbol[3:]  # 外卦
    nei = symbol[:3]  # 内卦

    wai, nei = const.YAOS.index(wai), const.YAOS.index(nei)

    gan = const.NAJIA[nei][0][0]
    ngz = [f'{gan}{zhi}' for zhi in const.NAJIA[nei][0][1:]]  # 排干支

    gan = const.NAJIA[wai][1][0]
    wgz = [f'{gan}{zhi}' for zhi in const.NAJIA[wai][1][1:]]  # 排干支

    return ngz + wgz


def Qin6(w1, w2):
    """
    两个五行判断六亲
    水1 # 木2 # 金3 # 火4 # 土5

    :param w1:
    :param w2:
    :return:
    """
    w1 = const.XING5.index(w1) if type(w1) is str else w1
    w2 = const.XING5.index(w2) if type(w2) is str else w2

    ws = w1 - w2
    ws = ws + 5 if ws < 0 else ws
    q6 = const.QING6[ws]

    logger.debug(ws)
    logger.debug(q6)

    return q6



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['najia']

package_data = \
{'': ['*'], 'najia': ['data/*']}

install_requires = \
['arrow>=1.2.3,<2.0.0',
 'click>=8.1.3,<9.0.0',
 'jinja2>=3.1.2,<4.0.0',
 'sxtwl<2',
 'tzlocal>=4.3,<5.0']

setup_kwargs = {
    'name': 'najia',
    'version': '0.1.1',
    'description': '',
    'long_description': "# 纳甲六爻排盘项目\n\n[![image](https://img.shields.io/pypi/v/najia.svg)](https://pypi.python.org/pypi/najia)\n\n[![image](https://img.shields.io/travis/bopo/najia.svg)](https://travis-ci.org/bopo/najia)\n\n[![Documentation Status](https://readthedocs.org/projects/najia/badge/?version=latest)](https://najia.readthedocs.io/en/latest/?badge=latest)\n\n[![Updates](https://pyup.io/repos/github/bopo/najia/shield.svg)](https://pyup.io/repos/github/bopo/najia/)\n\nPython Boilerplate contains all the boilerplate you need to create a\nPython package.\n\n-   Free software: MIT license\n-   Documentation: <https://najia.readthedocs.io>.\n\n## Features\n\n-   全部安易卦爻\n-   函数独立编写\n-   测试各个函数\n-   重新命名函数\n\n阳历，阴历（干支，旬空）\n\n-   卦符: mark (001000)，自下而上\n-   卦名: name\n-   变爻: bian\n-   卦宫: gong\n-   六亲: qin6\n-   六神: god6\n-   世爻: shiy, ying\n-   纳甲: naja\n-   纳甲五行: dzwx\n-   卦宫五行: gowx\n\n## 修复问题\n\n-   解决: 六神不对\n-   解决: 世应也有点小BUG , 地天泰卦的世爻为3, 应爻为6\n-   解决: 归魂卦世爻为3 此处返回4, 需要修改\n\n\\* 解决: 归魂卦的六亲是不对的,原因是utils.py里\n判断六爻卦的卦宫名时,优先判读了if index in (1, 2, 3, 6)\n而归魂卦的世爻也在3爻,被这个条件带走了. 解决: elif hun==\\'归魂\\'\n这个条件调到前面即可 \\* 解决: 还有一个不知是否算是错误的地方,就是bian\n变卦中的六亲,\n程序中是按变卦所在的本宫卦来定的,而不是按初始卦所属的本宫卦来定的六亲.\n\n## Credits\n\nThis package was created with\n[Cookiecutter](https://github.com/audreyr/cookiecutter) and the\n[audreyr/cookiecutter-pypackage](https://github.com/audreyr/cookiecutter-pypackage)\nproject template.\n",
    'author': 'bopo',
    'author_email': 'ibopo@126.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
